@@ -17,16 +17,18 @@ public :
 	 * getLoss evaluates the example loss.
 	 * The function returns the loss value
 	 */
-	//virtual double getLoss(example *&ec, gd_vars &vars) = 0;
-	virtual double getLoss(double prediction, double label) = 0;
+	//virtual float getLoss(example *&ec, gd_vars &vars) = 0;
+	virtual float getLoss(float prediction, float label) = 0;
 
 	/*
 	 * getUpdate evaluates the update scalar
 	 * The function return the update scalar
 	 */
-	//virtual double getUpdate(example *&ec, gd_vars &vars) = 0;
-	virtual double getUpdate(double prediction, double label) = 0;
-
+	virtual float getUpdate(float prediction, float label, float eta_t, float norm) = 0;
+	virtual float getRevertingWeight(float prediction, float eta_t) = 0;
+	virtual float getSquareGrad(float prediction, float label) = 0;
+	virtual float first_derivative(float prediction, float label) = 0;
+	virtual float second_derivative(float prediction, float label) = 0;
 	virtual ~loss_function() {};
 };
 
