@@ -23,7 +23,7 @@ void parse_regressor_args(po::variables_map& vm, regressor& r, string& final_reg
 
 void initialize_regressor(regressor &r);
 
-void finalize_regressor(ofstream& o, regressor &r);
-void dump_regressor(ofstream &o, regressor &r);
+void finalize_regressor(string reg_name, regressor &r);
+void dump_regressor(string reg_name, regressor &r);
 
 #endif
