@@ -13,8 +13,9 @@ void cache_simple_label(void* v, io_buf& cache);
 void default_simple_label(void* v);
 void parse_simple_label(void* v, v_array<substring>& words);
 void delete_simple_label(void* v);
+float get_weight(void* v);
 const label_parser simple_label = {default_simple_label, parse_simple_label, 
 				   cache_simple_label, read_cached_simple_label, 
-				   delete_simple_label, sizeof(label_data)};
+				   delete_simple_label, get_weight, sizeof(label_data)};
 
 #endif
